//=====================================================================
//	       .
//=====================================================================

//---------------------------------------------------------------------
#property copyright 	"Dima S., 2011."
#property link      	"dimascub@mail.com"
//---------------------------------------------------------------------

//=====================================================================
//	   :
//=====================================================================
class TBalanceHistory
  {
private:
   int               real_trades;                                     //    

public:
   string            trade_symbol;                                    //    

protected:
   //	"" :
   double            org_datetime_array[];                            // / 
   double            org_result_array[];                              //  

                                                                      //	     :
   double            group_datetime_array[];                          // / 
   double            group_result_array[];                            //  

   double            last_result_array[];                             //       (    OY )
   double last_datetime_array[ ];                                     //       (    OX )

private:
   void              SortMasterSlaveArray(double &_m[],double &_s[]); //      

public:
   void              SetTradeSymbol(string _symbol);                  // /  
   string            GetTradeSymbol();                                //   
   int               GetRealTrades();                                 //    
   int               GetTradeResultsArray();                          //    

public:
                     TBalanceHistory(string _symbol);                 // 
   void             ~TBalanceHistory();                               // 
  };
//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void  TBalanceHistory::TBalanceHistory(string _symbol)
   :
trade_symbol(_symbol)
  {
  }
//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void  TBalanceHistory::~TBalanceHistory()
  {
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
int  TBalanceHistory::GetRealTrades()
  {
   return(this.real_trades);
  }
//---------------------------------------------------------------------
//	/  :
//---------------------------------------------------------------------
void  TBalanceHistory::SetTradeSymbol(string _symbol)
  {
   this.trade_symbol=_symbol;
  }
//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
string  TBalanceHistory::GetTradeSymbol()
  {
   return(this.trade_symbol);
  }
//---------------------------------------------------------------------
//	       :
//---------------------------------------------------------------------
void  TBalanceHistory::SortMasterSlaveArray(double &_master[],double &_slave[])
  {
   int            size=ArraySize(_master);
   if(size!=ArraySize(_slave))
     {
      return;
     }

   double      temp_m=0.0;
   double      temp_s;

   for(int i=1;  i<=size; i++)
     {
      for(int j=0;  j<size-i;  j++)
        {
         //	    ,    :
         if(_master[j]<_master[j+1])
           {
            temp_m=_master[j];
            _master[j]=_master[j+1];
            _master[j+1]=temp_m;

            temp_s=_slave[j];
            _slave[j]=_slave[j+1];
            _slave[j+1]=temp_s;
           }
        }
     }
  }
//---------------------------------------------------------------------
//	     :
//---------------------------------------------------------------------
//	-    ;
//---------------------------------------------------------------------
int  TBalanceHistory::GetTradeResultsArray()
  {
   int            index,count;
   long         deal_type,deal_entry;
   int            deal_close_time,current_time;
   ulong         deal_ticket;                                                                  //  
   double      trade_result;
   string      deal_symbol;

   this.real_trades=0;

//	    ,     :
   if(this.trade_symbol==NULL)
     {
      return(0);
     }

//	     c     :
   if(HistorySelect(0,TimeCurrent())!=true)
     {
      return(0);
     }

//	  :
   count=HistoryDealsTotal();

//	    ,  ,    ,    :
   if(count<2)
     {
      return(0);
     }

//	 ,   "" :
   if(( ArraySize(this.org_datetime_array))!=count)
     {
      ArrayResize( this.org_datetime_array, count );
      ArrayResize( this.org_result_array, count );
     }

//	 ""     :
   for(index=count-1; index>=0; index--)
     {
      deal_ticket=HistoryDealGetTicket(index);

      //	    ,    :
      deal_entry=HistoryDealGetInteger(deal_ticket,DEAL_ENTRY);
      if(deal_entry!=DEAL_ENTRY_OUT)
        {
         continue;
        }

      //	  :
      deal_symbol=HistoryDealGetString(deal_ticket,DEAL_SYMBOL);
      if(deal_symbol!=this.trade_symbol)
        {
         continue;
        }

      //	  :
      deal_type=HistoryDealGetInteger(deal_ticket,DEAL_TYPE);
      if(deal_type!=DEAL_TYPE_BUY && deal_type!=DEAL_TYPE_SELL)
        {
         continue;
        }

      //	  :
      deal_close_time=(int)HistoryDealGetInteger(deal_ticket,DEAL_TIME);

      //	,    :
      this.org_datetime_array[this.real_trades]=(double)(deal_close_time/60);
      this.org_result_array[this.real_trades]=HistoryDealGetDouble(deal_ticket,DEAL_PROFIT)/HistoryDealGetDouble(deal_ticket,DEAL_VOLUME);
      this.real_trades++;
     }

//	  ,  ,    ,    :
   if(this.real_trades<2)
     {
      return(0);
     }
   count=this.real_trades;

//	 ""   /  :
   SortMasterSlaveArray(this.org_datetime_array,this.org_result_array);

//	 ,    :
   if(( ArraySize(this.group_datetime_array))!=count)
     {
      ArrayResize( this.group_datetime_array, count );
      ArrayResize( this.group_result_array, count );
     }
   ArrayInitialize( this.group_datetime_array, 0.0 );
   ArrayInitialize( this.group_result_array, 0.0 );

//	     (    /   ):
   this.real_trades=0;
   for(index=0; index<count; index++)
     {
      //	  :
      deal_close_time=(int)this.org_datetime_array[index];
      trade_result=this.org_result_array[index];

      //	 ,        :
      current_time=(int)this.group_datetime_array[this.real_trades];
      if(current_time>0 && (int)MathAbs(current_time-deal_close_time)>0)
        {
         this.real_trades++;                                    //     
         this.group_result_array[this.real_trades]=trade_result;
         this.group_datetime_array[this.real_trades]=deal_close_time;
        }
      else
        {
         this.group_result_array[this.real_trades]+=trade_result;
         this.group_datetime_array[this.real_trades]=deal_close_time;
        }
     }
   this.real_trades++;                                          //     

                                                                //	  ,  ,    ,    :
   if(this.real_trades<2)
     {
      return(0);
     }

   if(ArraySize(this.last_result_array)!=this.real_trades)
     {
      ArrayResize( this.last_result_array, this.real_trades );
      ArrayResize( this.last_datetime_array, this.real_trades );
     }

//	        :
   for(index=0; index<this.real_trades; index++)
     {
      this.last_result_array[this.real_trades-1-index]=this.group_result_array[index];
      this.last_datetime_array[this.real_trades-1-index]=this.group_datetime_array[index];
     }

//	         :
   for(index=1; index<this.real_trades; index++)
     {
      this.last_result_array[index]+=this.last_result_array[index-1];
     }

   return(this.real_trades);
  }
//=====================================================================
//	   :
//=====================================================================
class TBalanceSlope : public TBalanceHistory
  {
protected:
   double            current_slope;                                   //     
   double            current_sko;                                     // .      

private:
   double            LR_koeff_A,LR_koeff_B;                           //      
   double            LR_points_array[];                               //     

private:
   void              CalcLR(double &X[],double &Y[]);                 //     

public:
   double            CalcSlope();                                     //   
   double            GetCurrentSlope();                               //    
   double            GetCurrentSKO();                                 //    

public:
   void              TBalanceSlope(string _symbol);                   // 
   void             ~TBalanceSlope();                                 // 
  };
//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void  TBalanceSlope::TBalanceSlope(string _symbol)
   :
TBalanceHistory(_symbol)
  {
   this.current_slope=0.0;
   this.current_sko=0.0;
  }
//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void  TBalanceSlope::~TBalanceSlope()
  {
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TBalanceSlope::GetCurrentSlope()
  {
   return(this.current_slope);
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TBalanceSlope::GetCurrentSKO()
  {
   return(this.current_sko);
  }
//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
//	:   aka Prival,  Skype: privalov-sv
//---------------------------------------------------------------------
//	 :
//		X[ ] -       X;
//		Y[ ] -       Y;
//---------------------------------------------------------------------
void  TBalanceSlope::CalcLR(double &X[],double &Y[])
  {
   double      mo_X=0,mo_Y=0,var_0=0,var_1=0;
   int            i;
   int            size=ArraySize(X);
   double      nmb=(double)size;
   double      temp_y,temp_x;

//	    ,    :
   if(size<2)
     {
      return;
     }

   for(i=0; i<size; i++)
     {
      mo_X += X[ i ];
      mo_Y += Y[ i ];
     }
   mo_X /= nmb;
   mo_Y /= nmb;

   for(i=0; i<size; i++)
     {
      temp_x = X[ i ] - mo_X;
      temp_y = Y[ i ] - mo_Y;

      ////		var_0 += ( X[ i ] - mo_X ) * ( Y[ i ] - mo_Y );
      ////		var_1 += ( X[ i ] - mo_X ) * ( X[ i ] - mo_X );

      var_0 += temp_x * temp_y;
      var_1 += temp_x * temp_x;

      this.current_sko+=temp_y*temp_y;
     }

   this.current_sko=MathSqrt(this.current_sko)/nmb;

//	  A:
   if(var_1!=0.0)
     {
      this.LR_koeff_A=var_0/var_1;
     }
   else
     {
      this.LR_koeff_A=0.0;
     }

//	  B:
   this.LR_koeff_B=mo_Y-this.LR_koeff_A*mo_X;

//	  ,    :
   ArrayResize(this.LR_points_array,size);
   for(i=0; i<size; i++)
     {
      this.LR_points_array[i]=this.LR_koeff_A*X[i]+this.LR_koeff_B;
     }
  }
//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
double  TBalanceSlope::CalcSlope()
  {
//	      :
   int         nmb=this.GetTradeResultsArray();
   if(nmb<2)
     {
      return(0.0);
     }

//	      :
   this.CalcLR( last_datetime_array, last_result_array );
   this.current_slope=this.LR_koeff_A;

   return(this.current_slope);
  }
//---------------------------------------------------------------------
